! ############################################################
!            Satellite Data Simulation Unit v2
!                     -  SIMvisir -
!
! Visible/infrared Simulator
!
!                 Based on RSTAR6, modified by H. M. (May, 2009)
!                   Updated for SDSUv2.1.0  by H. M. (Aug, 2009)
!                       with mie_lut_vir thoroughly rewritten.
!
!
 Subroutine SIMvisir
!
! general program for calculating radiation fields in a-o-l system
!--- history
! 95. 2.20 aerosol growing with relative humidity (I. Lensky)
!     8.11 introduction of taumax
!          modified vlspcv
! 95. 9.15 re-organized by t. nakajima, a. higurashi
! 95.12.30 updated humidity, icn definitions in rstr5
! 96. 2. 7 registered aerd in lbr2
! 96. 3.11 allow non-spherical parameters
!     5. 5 add ncomp, change ints5b arguments
! 98. 1. 7
!   u10 > 0.01 for rstr and pstr.  u10 < 0.01 will have significant
!   error in radiances due to angular integration errors for surface
!   reflection.  recommend that u10 is set to be 0.01 for flat surface
!   case
! 07.11.30 rstar6 with new gas absorption k-distribution (M. Sekiguchi)
! 08. 2.19 added flxd00 (T. Nakajima)
! 08. 2.19 rstr6b without ifrh and trh (T. Nakajima)
! 08. 8.14 add dintvl (M.Sekiguchi)
! 08. 9. 5 
!---
   Use MAINparameters
   Use VIRparameters
   Implicit none
! for main
   integer:: iui=11,iuo=12,iua=13,iuk=14,ius=15,iug=16   !! file number
   character:: err*64      !! error code
   character:: fnm*100     !! file name
! for data
   integer:: isol  = 1     !! solar incident flag (0:night, 1:day)
   integer:: inda  = 1     !! radiance flag (0: flux only, 1: flux & radiance)
   integer:: indg          !! surface flag (0:Lambert, 1: Ocean)
   integer:: imthd = 2     !! flux/intensity calculation modes
   integer:: nda           !! number of stream
!
   integer:: na0   = 1     !! number of solar zenith angles
   real(8):: th0(kna0)     !! solar zenith angles (degree)
!
   integer:: na1   = 1     !! number of zenith angles of emerging radiance
   real(8):: th1(kna1u)    !! zenith angles of emerging radiances (degree)
   integer:: nfi   = 1     !! number of azimuthal angles
   real(8):: fi(knfi)      !! azimuthal angles (degree)
!
   integer:: nw0   = 1     !! number of sub-wavelength grids
   real(8):: rx(knw01) = (/ 0.,0. /) !! boundaries for sub-wavelength 
                                     !!  integration
   real(8):: rf(knw0)  = (/ 1. /)    !! integrated response function
!                   
   integer:: nw                      !! number of center wavelength
   real(8), dimension(:), allocatable :: &
             wl            !! center wavelength (micron)
   real(8), dimension(:), allocatable :: &
             dw            !! scaling factor to define scaled wavelengths
   real(8), dimension(:), allocatable :: &
             galb          !! ground albedo (indg=0)/ U10 [m/sec] (when indg=1)
!
   integer:: matm = 1      !! atmospheric number
   integer:: nln           !! number of sublayers
   integer:: nln1          !! nln + 1

   integer:: ipbf(knln1)   !! interface number to define sublayers
!  integer:: ifrh = 1      !! humidity flag (0: model, 1: use trh)
!  real(8):: trh           !! total relative humiditiy

   integer:: npoly = 1     !! number of particle polydispersions
   integer:: icn           !! unit of cnpt indicator
   real(8):: wlcn          !! scaling wavelength [cm] (icn=2 or 3)
 
   integer:: ncomp(kpoly)  !! number of particle models
   real(8):: cnpt(kpoly)   !! column total quantity
   integer:: mptc(kpoly,4) !! particle model number
   real(8):: vptc(kpoly,4) !! relative dry volume concentration for mixing

! for INTS5B
   integer:: nl            !! number of atmospheric layer 
   real(8):: alt(knl)      !! altitude [km]
   real(8):: prs(knl)      !! pressure [mb]
   real(8):: tmp(knl)      !! temperature [K]
   integer:: nmol          !! number of gases
   real(8):: cng(knl,knm0) !! gas concentration [ppmv]
   integer:: nptc          !! number of particle polydispersions
   real(8):: cnp(knl,kptc) !! dry volume concentration
   integer:: ispcvp(3,kptc) !! internal mixture of fundamental materials
   real(8):: rfracp(3,kptc) !! dry component volume fractions
   real(8):: asphr(3,kptc)  !! non-spherical parameters
   real(8):: rop(kptc)     !! particle density of dry mixture [g/cm3]
   real(8):: dryap(6,4,kptc) !! parameters to define volume size distribution
   integer:: nawp(kptc)    !! number of RH to define particle growth
   real(8):: awcrp(kaw,kptc,2) !! growth parameters
!
   integer:: nv            !! number of fundamental species
   integer:: nwlv          !! number of wavelenggths for rfi tables
   real(8):: wlv(kwlv)     !! wavelengths [micron] for rfi tables
   real(8):: rfi(kwlv,knv,2)  !! refractive index of fundamental materials
!
! for RSTR6B
   integer:: init          !! initialization flag for RSTR6B
   integer:: nch           !! number of channel (k-dist file)
   integer:: dintvl        !! inverse of interval (k-dist file)
!
! for output
   real(8):: thk0(knln,10)      !! calculated optical parameters
   real(8):: sol                !! solar incident irradiance [W/m2/micron]
   real(8):: flxd0(kna0,kntau)  !! downward flux [W/m2/micron]
   real(8):: flxu0(kna0,kntau)  !! upward flux [W/m2/micron]
   real(8):: flxd00(kna0,kntau) !! downward direct flux [W/m2/micron]
   real(8):: ai0(kna1u,kna0,knfi,kntau) !! radiance [W/m2/micron/str]
!
! for work
   integer:: nx,ny,i,j,l,iw   !! do-loop indicator
   integer:: ierr          !! error indicator
   integer:: lf            !! layer counter
!
! for wvcal
   integer:: m,ind
   real(8):: p1,t1,rh,ppmv,gm3,e,gm3s,es
!
   real(8):: tbb
   real   :: wind_ave
!--exec
!
! parameter range check
!
   If ( nlyr > knln ) then
      print *, 'ERROR (SMIvisir): knln must exceed nlyr in VIRparameters.'
      stop
   Endif
   If ( nlyr > kntau-1 ) then
      print *, 'ERROR (SMIvisir): kntau must exceed nlyr+1 in VIRparameters.'
      stop
   Endif
   If ( nang > knang ) then
      print *, 'ERROR (SMIvisir): knang must exceed nang in VIRparameters.'
      stop
   Endif
!
! When nighttime is specfied
!
   If ( znth_slr < 0. ) isol = 0
!
! Background type
! / Moved later for SDSU v2.1.0 /
!
!   If ( background == 'water' ) then
!      indg = 1
!   Else if ( background == 'land' ) then
!      indg = 0
!   Else
!      print *, 'ERROR (SIMvisir): background must be water or land.'
!      stop
!   Endif
!
! wavelength
!
   nw = mxwavel
   allocate ( wl(mxwavel), dw(mxwavel), galb(mxwavel) )
   wl = dble(wavel) ; dw = 0.d0
!
! Solar and observing angles
!
   th0 = dble(znth_slr) ; th1 = 180.d0 - dble(znth_obs) ; fi = dble(azmth)
!
! vertical layers
!
   nln = nlyr ; nln1=nln+1
   do l = 1, nln1
      ipbf(l) = nln1-l+1
   end do
!
! / removed for SDSU /
!  open(iuo,file='out')
   fnm='vis_IR/MLATMD'
   open(iua,file=fnm,status='old')
! / removed for SDSU /
!  fnm='AERDB'
!  open(ius,file=fnm,status='old')
!  fnm='KRNL.OUT'
!  open(iuk,file=fnm,status='old')
   nch=2
   dintvl=1.d3 !! the normal k-distribution grid is adopted for SDSU. !!
   fnm='gas/ckd.g.ch_2_1e3_'//lab_endian
   open(iug,file=fnm,status='old',access='direct',form='unformatted', &
        recl=4*kp*kt*nch*kmol1)
!
   allocate ( Ivir_out(ngridx,ngridy,mxwavel) )
   allocate ( prs_sdsu(nln1),tmp_sdsu(nln1),hgt_sdsu(nln1) )
   allocate ( rhm_sdsu(nln1),cng_sdsu(nln1,kmol) )
   allocate ( Wcw_sdsu(nln),Wci_sdsu(nln),Wrr_sdsu(nln) )
   allocate ( Wsn_sdsu(nln),Wgr_sdsu(nln),Whl_sdsu(nln) )
   allocate ( Ncw_sdsu(nln),Nci_sdsu(nln),Nrr_sdsu(nln) )
   allocate ( Nsn_sdsu(nln),Ngr_sdsu(nln),Nhl_sdsu(nln) )

! read simulation parameters
!
! initialization
!
   call ints5b(iua,ius,matm,nl,alt,prs,tmp,nmol,cng,nptc,cnp, &
               ispcvp,rfracp,asphr,rop,dryap,nawp,awcrp,nv,nwlv,wlv,rfi)
!
! Domain-averaged wind speed / added for SDSUv2.1.0 /
!
   wind_ave = sum  (wind_sfc,mask=wind_sfc>0.) / &
              count(         mask=wind_sfc>0.)
!
! wavelength loop
!
   print *, '/// Visible/infrared simulation ///'
   do iw=1,nw
!
      print '(A,I2,A,F6.3,A)', ' nwave: ',iw,' at ',wavel(iw),' micron'
!
      If ( lut ) Call open_lut_vir ( iw )
      nda = ngausp(iw)
!
! nx and ny loops 
!
      do nx = 1, ngridx
      do ny = 1, ngridy
!
! interface with SDSU variables
!
! - surface parameters
!
         tsk_sdsu = dble(tskin(nx,ny))
! / modified for SDSUv2.1.0 /
         If ( background=='land'  .or. &
              background=='mixed' .and. sfc_type(nx,ny)==1 ) then
            galb(iw) = dble(albd_visir(nx,ny,iw))
            indg = 0
         Else If ( background=='water' .or. &
              background=='mixed' .and. sfc_type(nx,ny)==0 ) then
!
! domain-averaged wind speed is used instead of local value
! to save computational time. SW radiance is much less sensitive
! to wind speed than to cloud properties.
!
            galb(iw) = dble(wind_ave)
            indg = 2
!           galb(iw) = dble(wind_sfc(nx,ny))
!           indg = 1
         Else
            print *, "ERROR (SIMvisir): illegal background or sfc_type"
            stop
         Endif
!
! Reset indg at the beginning of every snapshot
!
         If ( nx == 1 .and. ny == 1 .and. indg > 0 ) indg = 1
!
! - meteorological parameters
!
         prs_sdsu(1:nln1) = dble(pres_lev(nx,ny,0:nln))
         tmp_sdsu(1:nln1) = dble(temp_lev(nx,ny,0:nln))
         hgt_sdsu(1:nln1) = dble(hght_lev(nx,ny,0:nln))
         rhm_sdsu(2:nln1) = dble(rlhm_avg(nx,ny,1:nln))
         Wcw_sdsu(1:nln ) = dble(Wcldw_avg(nx,ny,1:nln))
         Wci_sdsu(1:nln ) = dble(Wcldi_avg(nx,ny,1:nln))
         Wrr_sdsu(1:nln ) = dble(Wrain_avg(nx,ny,1:nln))
         Wsn_sdsu(1:nln ) = dble(Wsnow_avg(nx,ny,1:nln))
         Wgr_sdsu(1:nln ) = dble(Wgrpl_avg(nx,ny,1:nln))
         Whl_sdsu(1:nln ) = dble(Whail_avg(nx,ny,1:nln))
         Ncw_sdsu(1:nln ) = dble(Ncldw_avg(nx,ny,1:nln))
         Nci_sdsu(1:nln ) = dble(Ncldi_avg(nx,ny,1:nln))
         Nrr_sdsu(1:nln ) = dble(Nrain_avg(nx,ny,1:nln))
         Nsn_sdsu(1:nln ) = dble(Nsnow_avg(nx,ny,1:nln))
         Ngr_sdsu(1:nln ) = dble(Ngrpl_avg(nx,ny,1:nln))
         Nhl_sdsu(1:nln ) = dble(Nhail_avg(nx,ny,1:nln))
!
! - gas concentration
!
         do i = 1, nmol
            do l = 1, nln1
               Call lintp8 ( knl,hgt_sdsu(l),alt,cng(:,i),cng_sdsu(l,i) )
            end do
         end do
!        
! - water vapor
!
         do l = 2, nln
            rhm_sdsu(l) = 0.5d0*(rlhm_avg(nx,ny,l) + rlhm_avg(nx,ny,l-1))
         end do
         rhm_sdsu(1) = rlhm_avg(nx,ny,1) ; rhm_sdsu(nln1) = rlhm_avg(nx,ny,nln)
         rhm_sdsu = rhm_sdsu * 1.d-2
!
         cng_sdsu(:,1) = 0.d0
         do l=1,nln
            p1=prs_sdsu(l)
            if(p1<50.d0) cycle
            t1=tmp_sdsu(l)
            rh=rhm_sdsu(l)
            call wvcal(1,p1,t1,rh,ppmv,gm3,e,gm3s,es)
            cng_sdsu(l,1)=ppmv
         enddo
!
! - radiative transfer calculation
!
         init=1 ; err=''
         call rstr6b(init,iuk,iug,indg,inda,imthd,isol,nda,na0,th0, &
                na1,th1,nfi,fi,wl(iw),dw(iw),nw0,rx,rf,galb(iw),icn, &
                wlcn,npoly,ncomp,mptc,vptc,cnpt,nl,alt,prs,tmp,nmol, &
                cng,nptc,cnp,ispcvp,rfracp,asphr,rop,dryap,nawp,awcrp, &
                nv,nwlv,wlv,rfi,nln,ipbf,nch,dintvl,sfc_type(nx,ny), &
                thk0,sol,flxd0,flxd00,flxu0,ai0,err)
         if(err/='') then
            write(*,*) 'ERROR (vis_IR/SIMvisir): ',err
            stop
         endif
!
! / added for SDSU /
         If ( tbform(iw) ) then
            do l = 1, nln1
               ai0(1,1,1,l) = tbb(wl(iw),ai0(1,1,1,l))
            end do
         endif
!
         Ivir_out(nx,ny,iw) = ai0(1,1,1,1)
!
      end do ! ends ngridy loop
      end do ! ends ngridx loop
!
      If ( lut ) then
!
         deallocate ( cextcw_lut, cscacw_lut, phsfcw_lut )
         deallocate ( cextrr_lut, cscarr_lut, phsfrr_lut )
         deallocate ( cextci_lut, cscaci_lut, phsfci_lut )
         deallocate ( cextsn_lut, cscasn_lut, phsfsn_lut )
         deallocate ( cextgr_lut, cscagr_lut, phsfgr_lut )
         deallocate ( cexthl_lut, cscahl_lut, phsfhl_lut )
         deallocate ( pts_hydro, pts_temp_w, pts_temp_i  )
         deallocate ( ptn_cldi, ptn_cldw, ptn_rain )
         deallocate ( ptn_snow, ptn_grpl, ptn_hail )
!
      Endif
!
   end do       ! ends wavelength loop
!
   deallocate ( prs_sdsu,tmp_sdsu,hgt_sdsu )
   deallocate ( rhm_sdsu,cng_sdsu )
   deallocate ( Wcw_sdsu,Wci_sdsu,Wrr_sdsu )
   deallocate ( Wsn_sdsu,Wgr_sdsu,Whl_sdsu )
   deallocate ( Ncw_sdsu,Nci_sdsu,Nrr_sdsu )
   deallocate ( Nsn_sdsu,Ngr_sdsu,Nhl_sdsu )
!
 end Subroutine SIMvisir


